/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef FRTOC_TIMES_HH
#define FRTOC_TIMES_HH

#include "Time.hh"
#include "Interval.hh"
#include <memory>
#include <vector>

class FrameF;

/**  %frtoc_times class to read the FrTOC file and return the time frame time 
  *  structure as a vector of Times and Intervals
  *  \version 1.0; November 19, 2022
  *  \author John Zweizig <john.zweizig@ligo.org>
  */
class frtoc_times {
public:
   /** define the time list data type.
    */
   typedef std::vector<std::pair<Time, Interval> > skeleton;
public:
   /** Default constructor.
    */
   frtoc_times(void) = default;

   /**  Constructor with a specified FrameF reader instance. The frame times
     *  are read from the TOC in the with the build_skeleton() method inside
     *  the constructor
     *  \param ffin A FrameF instance used to read the frame file.
     */
   frtoc_times(FrameF& ffin);

   /**  Constructor with a specified open std::istream instance. The frame 
     *  times are read from the TOC in the with the build_skeleton() method 
     *  inside the constructor
     *  \param in  An istream instance used to read the frame file.
     */
   frtoc_times(std::istream& in);

   /** Copy Constructor
    */ 
   frtoc_times(const frtoc_times& ftoc) = default;

   /** Move Constructor
    */ 
   frtoc_times(frtoc_times&& ftoc) = default;

   /** destructor
    */
   ~frtoc_times(void) = default;

   /**  Find the TOC and read enough of it to get the times and durations of
     *  all frames in the frame file. If build_skeleton is called a second 
     *  time, the time vector is replace by that for the new frame file.
     *  \param ffin An open FrameF instance used to read the frame file.
     */
   void build_skeleton(FrameF& ffin);

   /**  get the duration calculated from the start of the first frame to the 
     *  end of the last frame.
     *  \brief Calculate the file duration.
     *  \return duration of frame file.
     */
   Interval duration(void) const;

   /**  get the start time of the first frame in the file. The start time 
     *  is zero if no frames were in the TOC.
     *  \brief Calculate the start time.
     *  \return Start time
     */
   Time start(void) const;
   
   /**  Return a reference to the time vector.
     *  \return reference to the time vector.
     */
   const skeleton& time_vector(void) const;
   
private:
   skeleton frame_vector;
};

//======================================  inline methods.
inline Time 
frtoc_times::start(void) const {
   if (frame_vector.empty()) return Time(0);
   return frame_vector[0].first;
}

inline const frtoc_times::skeleton&
frtoc_times::time_vector(void) const {
   return frame_vector;
}

#endif // !defined(FRTOC_TIMES_HH)
